require 'nokogiri'
$dataset = 'guideposts'

namespace :guideposts do 

task :help do
puts "Options: 
  #{$dataset}:refresh_all     kompletně obnov všechno
  #{$dataset}:refresh_foreign kompletně obnov databazi z cizích dat
                   (= :download_foreign, :destroy_foreign, :import_foreign)
  #{$dataset}:refresh_osm     kompletně obnov OSM data
  #{$dataset}:set_neighbours  vyhledej dvojice co k sobe patri
"
end

task :refresh_all => [ :refresh_foreign, :refresh_osm, :refresh_production_env ]

task :refresh_production_env => [:wipe_cache, :assets_prec] do
  `sudo service nginx restart`
end

task :assets_prec => [ :environment ] do
  system('rake assets:precompile RAILS_ENV=production')
end

task :wipe_cache => [ :environment ] do
  dir = Rails.root.join("public", "cached_pages")
  FileUtils.remove_dir(dir) if Dir.exist?(dir)
  Dir.mkdir(dir)
  imgdir_out = Rails.root.join("public", "cached_pages", 'assets')
  imgdir_in = Rails.root.join('app','assets','images')
  Dir.mkdir(imgdir_out)
  FileUtils.cp(Dir.glob(imgdir_in+'*.png'), imgdir_out)
end

task :refresh_foreign => [ :download_foreign, :destroy_foreign, :import_foreign]
task :destroy_foreign => [:environment] do Point.where(typ: 'seznam').destroy_all end

task :download_foreign  do
  # ponechano tak, jelikoz se nestahuje a je v souboru "data/#{$dataset}/foreign-data.gpx"
end

task :import_foreign => [:environment] do
  puts 'Parsing foreign points datafile ...'
  doc = Nokogiri.XML(File.read("data/#{$dataset}/seznam-data.gpx"), nil, 'UTF-8')
  foreign_points = doc.css('wpt').map{ |src| SeznamPoint.new(src)}
    
  puts "Creating Seznam #{foreign_points.size} points in database ..."
  foreign_points.each do  |foreign_point|
    point = Point.new(foreign_point.def_hash)
    if !point.save
      raise "Error saving point #{foreign_point.name}"
    end
  end
  at_exit { set_neighbours }
end

class SeznamPoint # < Location
  def initialize(source_xml); @src=source_xml; end
  def lat; @src.attributes['lat'].value.to_f end
  def lon; @src.attributes['lon'].value.to_f end
  def tags; @src.css('extensions').first.element_children.map{|el| [el.name,el.content]}.to_h end
  def name; tags['TEXT1'] end
  def sid; tags['ID'].to_i end
  def def_hash; {lat: lat, lon: lon, name: name, src: @src.to_s, typ:'seznam'} end
end

task :refresh_osm => [ :download_osm, :destroy_osm, :import_osm]
task :destroy_osm => [:environment] do Point.where(typ: 'osm').destroy_all end

task :download_osm  do
  fileout = "data/#{$dataset}/osm_data.json"
  if !File.exists?(fileout) or (Time.now-File.mtime(fileout))>60*60  # stahovat jen jednou za hodinu
    district = 'Q213'  # Czech republic
    # district = 'Q1085'  # Prague
    # district = 'Q3489926'
    `wget -O #{fileout}  'http://overpass-api.de/api/interpreter?data=[out:json];area[wikidata=\"#{district}\"];(node[information=guidepost][hiking=yes](area););out;'`
  end
end

task :import_osm => [:environment] do
  puts "Parsing OSM JSON file ..."
  osm_points = JSON.parse(File.read("data/#{$dataset}/osm_data.json"))['elements'].
    select{ |osm_data| ['container',nil].include? osm_data['tags']['recycling_type'] }.
    map{ |source| OsmPoint.new(source)}
  puts "Creating OSM points in database ..."
  osm_points.each do  |osm_point|
    Point.new(osm_point.def_hash).save
  end
  at_exit { set_neighbours }
end

task :set_neighbours => [:environment] do
  at_exit { set_neighbours }
end

# nastavi near_id u vsech pointu
def set_neighbours
  return if $neighbours_already_set
  puts "Setting neighbours"
  Point.all.each do |point|
    near = point.near
    unless near.empty?
      point.near_id = near.first.id
      point.save
    end
  end
  $neighbours_already_set = true
end

class OsmPoint # < Location
  def initialize(source); @src=source end
  def lat; @src['lat'] end
  def lon; @src['lon'] end
  def tags; @src['tags'] end
  def osmid; @src['id'] end
  def name; tags['name'] || "osm##{osmid}"  end
  def def_hash; {lat: lat, lon: lon, name: name, src: @src.to_json, typ:'osm'} end
end

end # namespace